package com.utopiaindustries.hseobservationsapp.activities.OtherHSEActivityForms;

import android.content.Context;
import android.content.Intent;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.text.Editable;
import android.text.InputType;
import android.text.TextWatcher;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.inputmethod.EditorInfo;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import androidx.activity.EdgeToEdge;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.graphics.Insets;
import androidx.core.view.ViewCompat;
import androidx.core.view.WindowInsetsCompat;
import androidx.lifecycle.ViewModelProvider;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.utopiaindustries.hseobservationsapp.R;
import com.utopiaindustries.hseobservationsapp.activities.DashboardActivity;
import com.utopiaindustries.hseobservationsapp.activities.ObservationForms.ObservationTwoActivity;
import com.utopiaindustries.hseobservationsapp.helper.Helper;
import com.utopiaindustries.hseobservationsapp.helper.Preference;
import com.utopiaindustries.hseobservationsapp.utils.ProgressDialogFragment;
import com.utopiaindustries.hseobservationsapp.utils.StorageManager.HseReportRequest;
import com.utopiaindustries.hseobservationsapp.utils.StorageManager.ReportActivityModel;
import com.utopiaindustries.hseobservationsapp.utils.StorageManager.StorageManager;
import com.utopiaindustries.hseobservationsapp.viewmodels.LoginViewModel;

import java.util.List;

public class OtherHseActivity extends AppCompatActivity {

    Button btnSubmit;
    EditText etActivityName, etDescription;

    LoginViewModel loginViewModel;
    ImageView imgBack;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        EdgeToEdge.enable(this);
        setContentView(R.layout.activity_other_hse);
        ViewCompat.setOnApplyWindowInsetsListener(findViewById(R.id.main), (v, insets) -> {
            Insets systemBars = insets.getInsets(WindowInsetsCompat.Type.systemBars());
            v.setPadding(systemBars.left, systemBars.top, systemBars.right, systemBars.bottom);
            return insets;
        });

        initializeLayouts();

        imgBack.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                finish();
            }
        });

        etActivityName.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                StorageManager.getInstance().getOtherActivityModel().get(0).setActivityName(s.toString());
            }

            @Override
            public void afterTextChanged(Editable s) {

            }
        });

        etDescription.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                StorageManager.getInstance().getOtherActivityModel().get(0).setDescription(s.toString());
            }

            @Override
            public void afterTextChanged(Editable s) {

            }
        });

        btnSubmit.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (isValidate()) {
                    alertReportSubmit(OtherHseActivity.this);
                }
            }
        });
    }

    private void initializeLayouts() {

        imgBack = findViewById(R.id.img_back);
        etActivityName = findViewById(R.id.et_activityName);
        etDescription = findViewById(R.id.et_description);
        etDescription.setImeOptions(EditorInfo.IME_ACTION_DONE);
        etDescription.setRawInputType(InputType.TYPE_CLASS_TEXT);

        btnSubmit = findViewById(R.id.btn_submit);

        loginViewModel = new ViewModelProvider(this).get(LoginViewModel.class);

        loginViewModel.getLoadingState().observe(this, isLoading -> {

            if (isLoading != null && isLoading) {
                showProgressDialog();
            } else {
                dismissProgressDialog();
            }
        });

        loginViewModel.getErrorMessage().observe(this, errorResponse -> {
            if (errorResponse.isEmpty()) {
                Toast.makeText(this, "Something went wrong", Toast.LENGTH_SHORT).show();
            } else {
                Toast.makeText(this, errorResponse, Toast.LENGTH_SHORT).show();
            }
        });

        loginViewModel.getUserSaveLiveData().observe(this, hseSaveResponse -> {
            Log.e("Response: ",""+hseSaveResponse.toString());
            if (hseSaveResponse != null && hseSaveResponse.getResult().equalsIgnoreCase("success")) {
                StorageManager.getInstance().clearOtherActivityModule();
                Toast.makeText(this, "Report Submitted Successfully", Toast.LENGTH_SHORT).show();
                Intent intent = new Intent(OtherHseActivity.this, DashboardActivity.class);
                //intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP | Intent.FLAG_ACTIVITY_NEW_TASK);
                intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK);
                startActivity(intent);
                finish();
                overridePendingTransition(R.anim.slide_in_right, R.anim.slide_out_left);

            }
        });
    }

    public boolean isValidate() {
        boolean returnValue = true;
        String message = "";

        if (etDescription.getText().toString().isEmpty()) {
            message = "Please enter description.";
            returnValue = false;
        }

        if (etActivityName.getText().toString().isEmpty()) {
            message = "Please enter activity Name.";
            returnValue = false;
        }

        if (!returnValue) {
            Toast.makeText(this, message, Toast.LENGTH_SHORT).show();
        }

        return returnValue;
    }

    public void alertReportSubmit(Context con) {
        ViewGroup viewGroup = findViewById(android.R.id.content);

        TextView dialogOkBtn, dialogCancelBtn;

        AlertDialog.Builder builder = new AlertDialog.Builder(con);
        View view1 = LayoutInflater.from(con).inflate(R.layout.custom_layout_for_report_submission, viewGroup, false);
        builder.setCancelable(false);
        builder.setView(view1);

        dialogOkBtn = view1.findViewById(R.id.dialogOkBtn);
        dialogCancelBtn = view1.findViewById(R.id.dialogCancelBtn);

        AlertDialog alertDialog = builder.create();
        alertDialog.getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));

        dialogOkBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                alertDialog.dismiss();

                List<ReportActivityModel> reportList = StorageManager.getInstance().getOtherActivityModel();
                HseReportRequest<ReportActivityModel> reportRequest = new HseReportRequest<>();
                reportRequest.setObservation_date(StorageManager.getInstance().getObservation_date());
                reportRequest.setUserId(StorageManager.getInstance().getUserId());
                reportRequest.setSiteId(StorageManager.getInstance().getSite_id());
                reportRequest.setSupervisorId(StorageManager.getInstance().getSupervisorId());
                reportRequest.setSupervisorName(StorageManager.getInstance().getSupervisorName());
                reportRequest.setShift(StorageManager.getInstance().getShift());
                reportRequest.setDepartmentId(StorageManager.getInstance().getDepartmentId());
                reportRequest.setBuildingId(StorageManager.getInstance().getBuildingId());
                reportRequest.setFloorId(StorageManager.getInstance().getFloorId());
                reportRequest.setRecordTypeId(StorageManager.getInstance().getRecordTypeId());
                reportRequest.setCreatedBy(Preference.getMyStringPref(Helper.project_file, Helper.logInUser, OtherHseActivity.this));
                reportRequest.setReportData(reportList);

                /*Gson gson = new GsonBuilder().setPrettyPrinting().create();
                String jsonRequest = gson.toJson(reportRequest);
                Log.e("RequestModel JSON", jsonRequest);*/

                loginViewModel.saveHSEData(reportRequest);


            }
        });

        dialogCancelBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                alertDialog.dismiss();
                //Toast.makeText(con, "Cancel", Toast.LENGTH_SHORT).show();

            }
        });

        alertDialog.show();
    }

    public void showProgressDialog() {
        ProgressDialogFragment progressDialog = new ProgressDialogFragment();
        progressDialog.setCancelable(false);
        progressDialog.show(getSupportFragmentManager(), "progressDialog");
    }

    public void dismissProgressDialog() {
        ProgressDialogFragment progressDialog = (ProgressDialogFragment)
                getSupportFragmentManager().findFragmentByTag("progressDialog");
        if (progressDialog != null) {
            progressDialog.dismiss();
        }
    }


}