package com.utopiaindustries.hseobservationsapp.adapters;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.CheckBox;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.utopiaindustries.hseobservationsapp.R;
import com.utopiaindustries.hseobservationsapp.models.HseData.HsePtwType;

import java.util.ArrayList;
import java.util.List;

public class HsePtwTypeAdapter extends RecyclerView.Adapter<HsePtwTypeAdapter.ViewHolder> {

    private List<HsePtwType> list;
    private Context context;

    public HsePtwTypeAdapter(Context context, List<HsePtwType> list) {
        this.context = context;
        this.list = list;
    }

    public List<HsePtwType> getSelectedItems() {
        List<HsePtwType> selected = new ArrayList<>();
        for (HsePtwType item : list) {
            if (item.isSelected()) selected.add(item);
        }
        return selected;
    }

    @NonNull
    @Override
    public ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(context).inflate(R.layout.item_ptw_type, parent, false);
        return new ViewHolder(view);
    }

    @Override
    public void onBindViewHolder(@NonNull ViewHolder holder, int position) {
        HsePtwType item = list.get(position);
        holder.title.setText(item.getTitle());
        holder.checkbox.setChecked(item.isSelected());

        holder.checkbox.setOnCheckedChangeListener(null); // Clear previous listener

        holder.checkbox.setOnCheckedChangeListener((buttonView, isChecked) -> {
            int selectedCount = getSelectedItems().size();

            if (isChecked && selectedCount >= 2) {
                holder.checkbox.setChecked(false);
                Toast.makeText(context, "Only 2 selections allowed", Toast.LENGTH_SHORT).show();
            } else {
                item.setSelected(isChecked);
            }
        });

        holder.itemView.setOnClickListener(v -> {
            boolean newCheckedState = !holder.checkbox.isChecked();
            holder.checkbox.setChecked(newCheckedState);
        });
    }

    @Override
    public int getItemCount() {
        return list.size();
    }

    public static class ViewHolder extends RecyclerView.ViewHolder {
        CheckBox checkbox;
        TextView title;

        public ViewHolder(@NonNull View itemView) {
            super(itemView);
            checkbox = itemView.findViewById(R.id.checkbox);
            title = itemView.findViewById(R.id.title);
        }
    }
}

